/*
Copyright Rene Rivera 2013-2015
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)
*/

#if !defined(MSGPACK_PREDEF_OTHER_H) || defined(MSGPACK_PREDEF_INTERNAL_GENERATE_TESTS)
#ifndef MSGPACK_PREDEF_OTHER_H
#define MSGPACK_PREDEF_OTHER_H
#endif

#include <msgpack/predef/other/endian.h>
/*#include <msgpack/predef/other/.h>*/

#endif
